/* xdwapi sample 8

Function :
  Sets the attributes of headers, footers and page-numbers to the DocuWorks document and binder
  and updates them.
How to use :
  xdwsmpl8 document

  document : DocuWorks document (in).

  Creates DocuWorks binder with the same name of DocuWork document.
*/

#include <stdio.h>
#include <windows.h>

#include <xdw_api.h>
#include <xdwapian.h>

void print_error(int code)
{
	switch (code) {
	case XDW_E_NOT_INSTALLED:
		fprintf(stderr, "DocuWorks not installed.");
		break;
	case XDW_E_FILE_NOT_FOUND:
		fprintf(stderr, "File not found.");
		break;
	case XDW_E_ACCESSDENIED:
	case XDW_E_INVALID_NAME:
	case XDW_E_BAD_NETPATH:
		fprintf(stderr, "File not opened.");
		break;
	case XDW_E_BAD_FORMAT:
		fprintf(stderr, "File bad format.");
		break;
	case XDW_E_INVALID_ACCESS:
		fprintf(stderr, "Invalid access.");
		break;
	case XDW_E_FILE_EXISTS:
		fprintf(stderr, "File exists.");
		break;
	default:
		fprintf(stderr, "Error.");
		break;
	}
}

int check_document_type(XDW_DOCUMENT_HANDLE h)
{
	XDW_DOCUMENT_INFO info = { sizeof(XDW_DOCUMENT_INFO), 0, 0, 0, 0, 0, 0, 0, 0, 0 };
	int api_result = XDW_GetDocumentInformation(h, &info);
	if (api_result < 0) return api_result;

	if (info.nDocType != XDW_DT_DOCUMENT) {
		fprintf(stderr, "Set DocuWorks document to the file.");
		return XDW_E_INVALIDARG;
	}

	return api_result;
}

int set_pageform_attribute(XDW_DOCUMENT_HANDLE h)
{
	XDW_DOCUMENT_INFO info = { sizeof(XDW_DOCUMENT_INFO), 0, 0, 0, 0, 0, 0, 0, 0, 0 };
	int api_result = XDW_GetDocumentInformation(h, &info);
	if (api_result < 0) return api_result;

	char string[100];
	int alignment = 0;
	if (info.nDocType == XDW_DT_DOCUMENT) {
		strcpy(string, "page number # in DocuWorks document") ;
		// In case of DocuWorks document, set page-number at the lower left of the page.
		alignment = XDW_ALIGN_LEFT;
	} else {
		strcpy(string, "page number # in DocuWorks binder") ;
		// In case of DocuWorks document, set page-number at the lower right of the page.
		alignment = XDW_ALIGN_RIGHT;
	}

	//Set the content of the page-number.
	api_result = XDW_SetPageFormAttribute(h, XDW_PAGEFORM_PAGENUMBER, XDW_ATN_Text, XDW_ATYPE_STRING, string, NULL, NULL);
	if (api_result < 0) return api_result;

	//Set the alignment of the page-number.
	api_result = XDW_SetPageFormAttribute(h, XDW_PAGEFORM_PAGENUMBER, XDW_ATN_Alignment, XDW_ATYPE_INT, (char*)&alignment, NULL, NULL);

	return api_result;
}

int main(int argc, char** argv)
{
	int api_result = 0;

	if (argc != 2) {
		fprintf(stderr, "How to use : xdwsmpl8 document");
		return 0;
	}

	XDW_DOCUMENT_HANDLE h = NULL;
	XDW_OPEN_MODE_EX mode = {
		sizeof(XDW_OPEN_MODE_EX), XDW_OPEN_UPDATE, XDW_AUTH_NODIALOGUE };

	// Open the document handle of DocuWorks document.
	char in_path[_MAX_PATH];
	_fullpath(in_path, argv[1], _MAX_PATH);
	api_result = XDW_OpenDocumentHandle(in_path, &h, (XDW_OPEN_MODE*)&mode);
	if (api_result < 0) {
		print_error(api_result);
		return 0;
	}

	// Check the document type.
	api_result = check_document_type(h);
	if (api_result < 0) {
		return 0;
	}

	// Set the attributes of headers, footers and page-numbers.
	api_result = set_pageform_attribute(h);
	if (api_result < 0) {
		print_error(api_result);
		return 0;
	}

	// Update headers, footers, and page-numbers.
	api_result = XDW_UpdatePageForm(h, XDW_PAGEFORM_STAY, NULL);
	if (api_result < 0) {
		print_error(api_result);
		return 0;
	}

	// Reflect the changes to the document.
	api_result = XDW_SaveDocument(h, NULL);
	if (api_result < 0) {
		print_error(api_result);
		return 0;
	}

	// Close the document handle.
	XDW_CloseDocumentHandle(h, NULL);

	// Create DocuWorks binder with the same name of DocuWork document.
	char binder_path[_MAX_PATH];
	char name[_MAX_PATH], ext[_MAX_PATH];
	_splitpath(in_path, NULL, NULL, name, ext);
	strcat(name, ".xbd");
	_fullpath(binder_path, name, _MAX_PATH);
	if (GetFileAttributes(binder_path) != -1) {
		print_error(XDW_E_FILE_EXISTS);
		return 0;
	}
	api_result = XDW_CreateBinder(binder_path, NULL, NULL);
	if (api_result < 0) {
		print_error(api_result);
		return 0;
	}

	// Open the document handle of the binder.
	api_result = XDW_OpenDocumentHandle(binder_path, &h, (XDW_OPEN_MODE*)&mode);
	if (api_result < 0) {
		print_error(api_result);
		return 0;
	}

	// Insert the document to the binder.
	api_result = XDW_InsertDocumentToBinder(h, 1, in_path, NULL);
	if (api_result < 0) {
		print_error(api_result);
		return 0;
	}

	// Set the attributes of headers, footers and page-numbers.
	api_result = set_pageform_attribute(h);
	if (api_result < 0) {
		print_error(api_result);
		return 0;
	}

	// Update headers, footers, and page-numbers.
	// If the second argument is set XDW_PAGEFORM_REMOVE,
	// headers, footers and page-numbers of the internal document are removed.
	api_result = XDW_UpdatePageForm(h, XDW_PAGEFORM_STAY, NULL);
	if (api_result < 0) {
		print_error(api_result);
		return 0;
	}

	// Reflect the changes to the binder.
	api_result = XDW_SaveDocument(h, NULL);
	if (api_result < 0) {
		print_error(api_result);
		return 0;
	}

	// Close the document handle.
	XDW_CloseDocumentHandle(h, NULL);

	if (api_result < 0) return 0;

	return 1;
}