/* xdwapi sample 9

Function :
  Sets the document attributes for DocuWorks document.
  Attaches the label annotation at each page.
  Succeeds attributes of the DocuWorks document to another one.
  Gets the document attributes from another one.

How to use :
  xdwsmpl9 document1 document2

  document1 : DocuWorks document (in).
  document2 : DocuWorks document (in).
*/

using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using FujiXerox.DocuWorks.Toolkit;

namespace XdwSmpl9
{
    class Program
    {
        static int Main(string[] args)
        {
            int api_result = 0;

            if (args.Length != 2) {
                TextWriter errorWriter = Console.Error;
                errorWriter.WriteLine("How to use : xdwsmpl9 document1 document2");
                return 0;
            }

            Xdwapi.XDW_DOCUMENT_HANDLE handle = new Xdwapi.XDW_DOCUMENT_HANDLE();
            Xdwapi.XDW_OPEN_MODE_EX mode = new Xdwapi.XDW_OPEN_MODE_EX();
            mode.Option = Xdwapi.XDW_OPEN_UPDATE;
            mode.AuthMode = Xdwapi.XDW_AUTH_NODIALOGUE;

            // Open the document handle of document1.
            string in_path = Path.GetFullPath(args[0]);
            api_result = Xdwapi.XDW_OpenDocumentHandle(in_path, ref handle, mode);
            if (api_result < 0) {
                print_error(api_result);
                return 0;
            }

            // Set the document attributes in document1.
            api_result = set_document_attribute(handle);
            if (api_result < 0) {
                print_error(api_result);
                Xdwapi.XDW_CloseDocumentHandle(handle);
                return 0;
            }

            // Attach the label annotation.
            api_result = add_annotation(handle);
            if (api_result < 0) {
                print_error(api_result);
                Xdwapi.XDW_CloseDocumentHandle(handle);
                return 0;
            }

            // Reflect the changes to document1.
            api_result = Xdwapi.XDW_SaveDocument(handle);
            if (api_result < 0) {
                print_error(api_result);
                Xdwapi.XDW_CloseDocumentHandle(handle);
                return 0;
            }

            // Close the document handle.
            Xdwapi.XDW_CloseDocumentHandle(handle);

            // Open the document handle of document2.
            string in_path2 = Path.GetFullPath(args[1]);
            api_result = Xdwapi.XDW_OpenDocumentHandle(in_path2, ref handle, mode);
            if (api_result < 0) {
                print_error(api_result);
                return 0;
            }

            // Succeed the attributes from document1 to document2.
            api_result = succeed_attribute(handle, in_path);
            if (api_result < 0) {
                print_error(api_result);
                Xdwapi.XDW_CloseDocumentHandle(handle);
                return 0;
            }

            // Get the document attributes in document2.
            api_result = get_document_attribute(handle);
            if (api_result < 0) {
                print_error(api_result);
                Xdwapi.XDW_CloseDocumentHandle(handle);
                return 0;
            }

            // Reflect the changes to document2.
            api_result = Xdwapi.XDW_SaveDocument(handle);
            if (api_result < 0) {
                print_error(api_result);
                Xdwapi.XDW_CloseDocumentHandle(handle);
                return 0;
            }

            // Close the document handle.
            Xdwapi.XDW_CloseDocumentHandle(handle);

            if (api_result < 0)
                return 0;

            return 1;
        }

        static int add_annotation(Xdwapi.XDW_DOCUMENT_HANDLE handle)
        {
            // Use XDW_GetDocumentInformation to get the number of total pages.
            Xdwapi.XDW_DOCUMENT_INFO info = new Xdwapi.XDW_DOCUMENT_INFO();
            int api_result = Xdwapi.XDW_GetDocumentInformation(handle, ref info);
            if (api_result < 0)
                return api_result;
            int last_page = info.Pages;

            Xdwapi.XDW_ANNOTATION_HANDLE annotation = new Xdwapi.XDW_ANNOTATION_HANDLE();
            int x = 500;
            int y = 500;
            Xdwapi.XDW_AA_FUSEN_INITIAL_DATA label = new Xdwapi.XDW_AA_FUSEN_INITIAL_DATA();
            label.Width = 7500;
            label.Height = 2500;
            int color = Xdwapi.XDW_COLOR_FUSEN_YELLOW;

            // Attach a label annotation at each page
            for (int i = 1; i <= last_page; i++)
            {
                api_result = Xdwapi.XDW_AddAnnotation(
                    handle, Xdwapi.XDW_AID_FUSEN, i, x, y, label, ref annotation);
                if (api_result < 0)
                    break;

                // Set the attribute of the label annotation color.
                api_result = Xdwapi.XDW_SetAnnotationAttribute(
                    handle, annotation, Xdwapi.XDW_ATN_FillColor, Xdwapi.XDW_ATYPE_INT, color);

                if (api_result < 0)
                    break;
            }

            return api_result;
        }

        static int set_document_attribute(Xdwapi.XDW_DOCUMENT_HANDLE handle)
        {
            int api_result = 0;

            string out_string = null;
            // Set "Title"(Summary) as the document attribute.
            out_string = "xdwapi sample 9";
            api_result = Xdwapi.XDW_SetDocumentAttribute(handle, Xdwapi.XDW_PROP_TITLE, Xdwapi.XDW_ATYPE_STRING, out_string);
            if (api_result < 0)
                return api_result;

            // Set "Owner"(Custom) as the document attribute.
            out_string = "ABCxyz";
            api_result = Xdwapi.XDW_SetDocumentAttribute(handle, "User", Xdwapi.XDW_ATYPE_STRING, out_string);
            if (api_result < 0) return api_result;

            // "Document number"(Custom) as the document attribute.
            int number = 1234567890;
            api_result = Xdwapi.XDW_SetDocumentAttribute(handle, "Document number", Xdwapi.XDW_ATYPE_INT, number);

            return api_result;
        }

        static int succeed_attribute(Xdwapi.XDW_DOCUMENT_HANDLE handle, string file_path)
        {
            // Set the attribute to be succeeded.
            int succession = Xdwapi.XDW_SUMMARY_INFO | Xdwapi.XDW_USER_DEF | Xdwapi.XDW_ANNOTATION;

            // Succeed the attributes.
            int api_result = Xdwapi.XDW_SucceedAttribute(handle, file_path, 0, succession);

            return api_result;
        }

        static int get_document_attribute(Xdwapi.XDW_DOCUMENT_HANDLE handle)
        {
            int api_result = 0;
            string attribute_name = null;
            int attribute_type = 0;

            // Get the number of document attributes set.
            int attribute_num = Xdwapi.XDW_GetDocumentAttributeNumber(handle);
            for (int i = 1; i <= attribute_num; i++) {
                // Get the document attribute.
                Object attribute_value = null;
                api_result = Xdwapi.XDW_GetDocumentAttributeByOrder(handle, i, ref attribute_name, ref attribute_type, ref attribute_value);
                if (api_result < 0)
                    break;

                if (attribute_name.Equals(Xdwapi.XDW_PROP_TITLE)){
                    Console.WriteLine(" Title is {0}.", attribute_value);
                } else {
                    if (attribute_type == Xdwapi.XDW_ATYPE_STRING)
                    {
                        Console.WriteLine(" {0} is {1}.", attribute_name, attribute_value);
                    }
                    else if (attribute_type == Xdwapi.XDW_ATYPE_INT)
                    {
                        Console.WriteLine(" {0} is {1}.", attribute_name, (int)attribute_value);
                    }
                }
            }

            return api_result;
        }

        static void print_error(int code)
        {
            TextWriter errorWriter = Console.Error;
            switch (code) {
                case Xdwapi.XDW_E_NOT_INSTALLED:
                    errorWriter.WriteLine("DocuWorks not installed.");
                    break;
                case Xdwapi.XDW_E_FILE_NOT_FOUND:
                    errorWriter.WriteLine("File not found.");
                    break;
                case Xdwapi.XDW_E_ACCESSDENIED:
                case Xdwapi.XDW_E_INVALID_NAME:
                case Xdwapi.XDW_E_BAD_NETPATH:
                    errorWriter.WriteLine("File not opened.");
                    break;
                case Xdwapi.XDW_E_BAD_FORMAT:
                    errorWriter.WriteLine("File bad format.");
                    break;
                case Xdwapi.XDW_E_INVALID_ACCESS:
                    errorWriter.WriteLine("Invalid access.");
                    break;
                case Xdwapi.XDW_E_OUTOFMEMORY:
                    errorWriter.WriteLine("Out of memory.");
                    break;
                default:
                    errorWriter.WriteLine("Error.");
                    break;
            }
        }

    }
}
